@@ -16,11 +16,18 @@
  */
 
 #include "capturedevicecontroller.h"
-#include "qtgstreamercapturebackend.h"
-#include <QUrl>
-#include <QStringList>
+#include "capturebackendinterface.h"
+#include "backendinterface.h"
 #include "libsound_debug.h"
 
+#include <QCoreApplication>
+#include <QPluginLoader>
+#include <QStringList>
+
+#include <KPluginLoader>
+#include <KPluginFactory>
+#include <KPluginMetaData>
+
 /**
  * \class CaptureDeviceControllerPrivate
  * \internal
@@ -35,14 +42,48 @@ class CaptureDeviceControllerPrivate
 public:
     CaptureDeviceControllerPrivate(QObject *parent)
         : m_parent(parent)
+        , m_backend(nullptr)
         , m_initialized(false)
     {
+        QStringList dirsToCheck;
+        foreach (const QString &directory, QCoreApplication::libraryPaths()) {
+            dirsToCheck << directory + "/artikulate/libsound";
+        }
+        // load plugins
+        QPluginLoader loader;
+        foreach (const QString &dir, dirsToCheck) {
+            QVector<KPluginMetaData> metadataList = KPluginLoader::findPlugins(dir,
+                [=](const KPluginMetaData &data)
+            {
+                return data.serviceTypes().contains("artikulate/libsound/backend");
+            });
+
+            foreach (const auto &metadata, metadataList) {
+                loader.setFileName(metadata.fileName());
+                qCDebug(LIBSOUND_LOG) << "Load Plugin: " << metadata.name();
+                if (!loader.load()) {
+                    qCCritical(LIBSOUND_LOG) << "Error while loading plugin: " << metadata.name();
+                }
+                KPluginFactory *factory = KPluginLoader(loader.fileName()).factory();
+                if (!factory) {
+                    qCCritical(LIBSOUND_LOG) << "Could not load plugin: " << metadata.name();
+                    continue;
+                }
+                BackendInterface *plugin = factory->create<BackendInterface>(parent, QList< QVariant >());
+                if (plugin->captureBackend()) {
+                    m_backendList.append(plugin->captureBackend());
+                }
+            }
+        }
+        if (!m_backend && !m_backendList.isEmpty()) {
+            m_backend = m_backendList.first();
+        }
     }
 
     ~CaptureDeviceControllerPrivate()
     {
         delete m_backend;
-        m_backend = 0;
+        m_backend = nullptr;
     }
 
     void lazyInit()
@@ -50,18 +91,19 @@ public:
         if (m_initialized) {
             return;
         }
-        m_backend = new QtGStreamerCaptureBackend();
+        //TODO currently nothing to do
         m_initialized = true;
     }
 
-    QtGStreamerCaptureBackend * backend() const
+    CaptureBackendInterface * backend() const
     {
         Q_ASSERT(m_backend);
         return m_backend;
     }
 
     QObject *m_parent;
-    QtGStreamerCaptureBackend *m_backend;
+    CaptureBackendInterface *m_backend;
+    QList<CaptureBackendInterface *> m_backendList;
     bool m_initialized;
 };
 
